@@ -1,64 +1,31 @@
 from abc import abstractmethod
 from copy import copy, deepcopy
-from enum import Enum
 from itertools import chain
 from types import FunctionType
 from typing import Tuple, List, Union, Optional
+from os import linesep
 
 from pykusto.client import Table, KustoResponse
+from pykusto.enums import Order, Nulls, JoinKind, Distribution, BagExpansion
 from pykusto.expressions import BooleanType, ExpressionType, AggregationExpression, OrderedType, \
     StringType, AssignmentBase, AssignmentFromAggregationToColumn, AssignmentToSingleColumn, AnyTypeColumn, \
     BaseExpression, \
-    AssignmentFromColumnToColumn, AnyExpression, to_kql, expression_to_type, BaseColumn
+    AssignmentFromColumnToColumn, AnyExpression, to_kql, expression_to_type, BaseColumn, NumberType
 from pykusto.kql_converters import KQL
 from pykusto.logger import logger
 from pykusto.type_utils import KustoType, typed_column, plain_expression
 from pykusto.udf import stringify_python_func
 
 
-class Order(Enum):
-    ASC = "asc"
-    DESC = "desc"
-
-
-class Nulls(Enum):
-    FIRST = "first"
-    LAST = "last"
-
-
-class JoinKind(Enum):
-    INNERUNIQUE = "innerunique"
-    INNER = "inner"
-    LEFTOUTER = "leftouter"
-    RIGHTOUTER = "rightouter"
-    FULLOUTER = "fullouter"
-    LEFTANTI = "leftanti"
-    ANTI = "anti"
-    LEFTANTISEMI = "leftantisemi"
-    RIGHTANTI = "rightanti"
-    RIGHTANTISEMI = "rightantisemi"
-    LEFTSEMI = "leftsemi"
-    RIGHTSEMI = "rightsemi"
-
-
-class Distribution(Enum):
-    SINGLE = 'single'
-    PER_NODE = 'per_node'
-    PER_SHARD = 'per_shard'
-
-
-class BagExpansion(Enum):
-    BAG = "bag"
-    ARRAY = "array"
-
-
 class Query:
     _head: Optional['Query']
     _table: Optional[Table]
+    _table_name: Optional[str]
 
     def __init__(self, head=None) -> None:
         self._head = head if isinstance(head, Query) else None
         self._table = head if isinstance(head, Table) else None
+        self._table_name = head if isinstance(head, str) else None
 
     def __add__(self, other: 'Query') -> 'Query':
         self_copy = deepcopy(self)
@@ -117,11 +84,11 @@ class Query:
     def project_away(self, *columns: StringType) -> 'ProjectAwayQuery':
         return ProjectAwayQuery(self, columns)
 
-    def distinct(self, *columns: BaseExpression) -> 'DistinctQuery':
+    def distinct(self, *columns: BaseColumn) -> 'DistinctQuery':
         return DistinctQuery(self, columns)
 
     def distinct_all(self) -> 'DistinctQuery':
-        return DistinctQuery(self, (AnyExpression(KQL("*")),))
+        return DistinctQuery(self, (AnyTypeColumn(KQL("*")),))
 
     def extend(self, *args: Union[BaseExpression, AssignmentBase], **kwargs: ExpressionType) -> 'ExtendQuery':
         return ExtendQuery(self, *self.extract_assignments(*args, **kwargs))
@@ -176,7 +143,10 @@ class Query:
     def _compile_all(self, use_full_table_name) -> KQL:
         if self._head is None:
             if self._table is None:
-                return KQL("")
+                if self._table_name is None:
+                    return KQL("")
+                else:
+                    return KQL(self.get_table_name())
             else:
                 table = self._table
                 if use_full_table_name:
@@ -192,11 +162,23 @@ class Query:
         else:
             return self._head.get_table()
 
+    def get_table_name(self) -> str:
+        if self._head is None:
+            return self._table_name
+        else:
+            return self._head.get_table_name()
+
     def render(self, use_full_table_name: bool = False) -> KQL:
         result = self._compile_all(use_full_table_name)
         logger.debug("Complied query: " + result)
         return result
 
+    def pretty_render(self, use_full_table_name: bool = False) -> KQL:
+        kql = self.render(use_full_table_name)
+        if kql is not None:
+            kql = KQL(kql.replace(" |", linesep + "|"))
+        return kql
+
     def execute(self, table: Table = None) -> KustoResponse:
         if self.get_table() is None:
             if table is None:
@@ -267,16 +249,62 @@ class ProjectAwayQuery(Query):
 
 
 class DistinctQuery(Query):
-    _columns: Tuple[BaseExpression, ...]
+    _columns: Tuple[BaseColumn, ...]
 
-    def __init__(self, head: 'Query', columns: Tuple[BaseExpression]) -> None:
+    def __init__(self, head: 'Query', columns: Tuple[BaseColumn]) -> None:
         super().__init__(head)
         self._columns = columns
 
+    def sample(self, number_of_values: NumberType) -> 'SampleDistinctQuery':
+        """
+        https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/sampledistinctoperator
+        """
+        assert len(self._columns) == 1, "sample-distinct supports only one column"
+        return SampleDistinctQuery(self._head, self._columns[0], number_of_values)
+
+    def top_hitters(self, number_of_values: NumberType) -> 'TopHittersQuery':
+        """
+        https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/tophittersoperator
+        """
+        assert len(self._columns) == 1, "top-hitters supports only one column"
+        return TopHittersQuery(self._head, self._columns[0], number_of_values)
+
     def _compile(self) -> KQL:
         return KQL(f"distinct {', '.join(c.kql for c in self._columns)}")
 
 
+class SampleDistinctQuery(Query):
+    _number_of_values: NumberType
+    _column: BaseColumn
+
+    def __init__(self, head: 'Query', column: BaseColumn, number_of_values: NumberType) -> None:
+        super().__init__(head)
+        self._column = column
+        self._number_of_values = number_of_values
+
+    def _compile(self) -> KQL:
+        return KQL(f"sample-distinct {to_kql(self._number_of_values)} of {self._column.kql}")
+
+
+class TopHittersQuery(Query):
+    _number_of_values: NumberType
+    _column: BaseColumn
+    _by_expression: Optional[NumberType]
+
+    def __init__(self, head: 'Query', column: BaseColumn, number_of_values: NumberType, by_expression: Optional[NumberType] = None) -> None:
+        super().__init__(head)
+        self._column = column
+        self._number_of_values = number_of_values
+        self._by_expression = by_expression
+
+    def by(self, by_expression: NumberType) -> 'TopHittersQuery':
+        assert self._by_expression is None, "duplicate 'by' clause"
+        return TopHittersQuery(self._head, self._column, self._number_of_values, by_expression)
+
+    def _compile(self) -> KQL:
+        return KQL(f"top-hitters {to_kql(self._number_of_values)} of {self._column.kql}{'' if self._by_expression is None else f' by {to_kql(self._by_expression)}'}")
+
+
 class ExtendQuery(Query):
     _assignments: Tuple[AssignmentBase, ...]
 
