@@ -454,6 +454,12 @@ class TestFunction(TestBase):
             Query().extend(foo=f.split("1_2", "_")[3]).render()
         )
 
+    def test_count_of(self):
+        self.assertEqual(
+            ' | where (countof(stringField, "abc", "normal")) == 2',
+            Query().where(f.count_of(t.stringField, "abc") == 2).render()
+        )
+
     def test_tobool(self):
         self.assertEqual(
             " | where tobool(stringField)",
@@ -524,6 +530,12 @@ class TestFunction(TestBase):
             Query().summarize(f.any(t.stringField, t.numField, t.boolField)).render()
         )
 
+    def test_any_if(self):
+        self.assertEqual(
+            " | summarize anyif(stringField, boolField)",
+            Query().summarize(f.any_if(t.stringField, t.boolField)).render()
+        )
+
     def test_aggregation_assign_to(self):
         self.assertEqual(
             " | summarize foo = any(stringField)",
@@ -711,6 +723,12 @@ class TestFunction(TestBase):
             Query().summarize(f.percentiles(t.numField, 5, 50, 95)).render()
         )
 
+    def test_percentiles_array(self):
+        self.assertEqual(
+            " | summarize percentiles_array(numField, 5, 50, 95)",
+            Query().summarize(f.percentiles_array(t.numField, 5, 50, 95)).render()
+        )
+
     def test_stdev(self):
         self.assertEqual(
             " | summarize stdev(numField)",
@@ -797,17 +815,11 @@ class TestFunction(TestBase):
 
     def test_iff_ambiguous_type(self):
         with self.assertLogs(logger, logging.WARN) as cm:
-            # We want to assert there are no warnings, but the 'assertLogs' method does not support that.
-            # Therefore, we are adding a dummy warning, and then we will assert it is the only warning.
-            logger.warn("Dummy warning")
             self.assertEqual(
                 " | project foo = iff(boolField, time(3.0:0:0.0), foo - bar)",
                 Query().project(foo=f.iff(t.boolField, timedelta(3), col.foo - col.bar)).render()
             )
-        self.assertEqual(
-            ["WARNING:pykusto:Dummy warning"],
-            cm.output,
-        )
+        self.assertEqual([], cm.output)
 
     def test_iif(self):
         # iif is just an alias to iff
@@ -905,3 +917,9 @@ class TestFunction(TestBase):
             ' | extend foo = array_split(arrayField, numField)',
             Query().extend(foo=f.array_split(t.arrayField, t.numField)).render()
         )
+
+    def test_ingestion_time(self):
+        self.assertEqual(
+            ' | extend ingestionTime = ingestion_time()',
+            Query().extend(ingestionTime=f.ingestion_time()).render()
+        )
